package nl.ordina.odag.ejb.test.client;

import inheritance.IBankAccount;
import inheritance.IBillingDetails;
import inheritance.ICreditCard;
import inheritance.singletable.BankAccount;
import inheritance.singletable.BillingDetails;
import inheritance.singletable.CreditCard;

import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import junit.framework.TestCase;

public class InheritanceTest extends TestCase {
	private EntityManager em;

	private static EntityManagerFactory emf = Persistence
			.createEntityManagerFactory("course");;

	private static final String OWNER = "MB";

	protected void setUp() throws Exception {
		super.setUp();
		System.out.println("Create the entity manager");
		em = emf.createEntityManager();

	}

	protected void tearDown() throws Exception {
		super.tearDown();
		if (em != null) {
			try {
				if (em.isOpen())
					em.close();
			} catch (Exception e) {
				e.printStackTrace();
				return;
			}

		}
	}

	public void testSingleTable() {
		CreditCard cc = new CreditCard();
		BankAccount account = new BankAccount();
		BankAccount account2 = new BankAccount();

		setCreditCardDetails(OWNER, cc);
		fillAccount(OWNER, account);
		fillAccount("OtHER", account2);

		persist(new Object[] { cc, account, account2 });

		List<IBillingDetails> details = getDetails("BillingDetailsSingle");

		System.out.println(details.size());
	}

	public void testJoined() {
		inheritance.joined.CreditCard cc = new inheritance.joined.CreditCard();
		inheritance.joined.BankAccount account = new inheritance.joined.BankAccount();
		inheritance.joined.BankAccount account2 = new inheritance.joined.BankAccount();

		setCreditCardDetails(OWNER, cc);
		fillAccount(OWNER, account);
		fillAccount("OtHER", account2);

		persist(new Object[] { cc, account, account2 });
		inheritance.joined.CreditCard card = em.find(inheritance.joined.CreditCard.class, cc.getId());
		inheritance.joined.BankAccount acc = em.find(inheritance.joined.BankAccount.class, account.getId());
		System.out.println("card: id=" + card.getId() + ", month=" + card.getExpMonth());
		System.out.println("account: id=" + account.getId() + ", swift=" + account.getBankSwift());
		
		List<IBillingDetails> details = getDetails("BillingDetailsJoined");

		System.out.println(details.size());

	}

	private List<IBillingDetails> getDetails(String abstractSchemaName) {
		EntityTransaction tran = em.getTransaction();
		tran.begin();
		String query = "select details from " + abstractSchemaName
				+ "  as details where details.owner = :owner";
		List<IBillingDetails> details = em.createQuery(query).setParameter(
				"owner", OWNER).getResultList();
		assertEquals(2, details.size());
		for (IBillingDetails detail : details) {
			if (detail instanceof IBankAccount) {
				System.out.println("Found bank account with id "
						+ detail.getId() + " " + detail.getClass().getName());
			} else if (detail instanceof ICreditCard) {
				System.out.println("Found credit card with id "
						+ detail.getId() + " " + detail.getClass().getName());
			}
		}
		tran.commit();
		return details;
	}

	private void persist(Object[] objects) {
		EntityTransaction transaction = em.getTransaction();

		transaction.begin();
		for (Object obj : objects) {
			em.persist(obj);
		}
		transaction.commit();
	}

	private void setCreditCardDetails(String owner, ICreditCard cc) {
		cc.setCreated(new Date());
		cc.setExpMonth("12");
		cc.setExpYear("2001");
		cc.setOwner(owner);
		cc.setType(0);
	}

	private void fillAccount(String owner, IBankAccount account) {
		account.setBankName("ABN AMRO");
		account.setBankSwift("ABN2A");
		account.setCreated(new Date());
		account.setNumber("503386334");
		account.setOwner(owner);
	}
}
